@@ -11,8 +11,9 @@ import attr
 from fixtup.entity.fixtup_process import FixtupProcess
 from fixtup.entity.fixture import Fixture, State
 from fixtup.entity.fixture_template import FixtureTemplate
-from fixtup.exceptions import PluginRuntimeError
+from fixtup.exceptions import PluginRuntimeError, HookRuntimeError
 from fixtup.hook.base import HookEngine, HookEvent
+from fixtup.lib.env import with_cwd
 from fixtup.lib.signal import register_signal_handler, unregister_signal_handler
 from fixtup.logger import get_logger
 from fixtup.plugin.base import PluginEngine, PluginEvent
@@ -37,16 +38,19 @@ class FixtureEngine:
             return
 
         try:
-            shutil.copytree(fixture_template.directory, fixture.directory)
-            # restore the directory after having removing the old one
-            os.chdir(fixture.directory)
+            if not fixture_template.mount_in_place:
+                shutil.copytree(fixture_template.directory, fixture.directory)
+                # restore the directory after having removing the old one
+                os.chdir(fixture.directory)
 
             self.plugin_engine.run(PluginEvent.mounting, fixture)
             self.hook_engine.run(HookEvent.mounting, fixture_template)
             self.store.fixture_mounted(fixture_template, fixture)
-        except PluginRuntimeError:
+        except (PluginRuntimeError, HookRuntimeError):
             self.plugin_engine.release(PluginEvent.unmounting, fixture)
-            if os.path.isdir(fixture.directory):
+            # When a fixture is mount in place, Fixtup should preserve the original
+            # when error happens
+            if not fixture_template.mount_in_place and os.path.isdir(fixture.directory):
                 shutil.rmtree(fixture.directory, True)
 
             raise
@@ -59,9 +63,12 @@ class FixtureEngine:
                 f"fixture {fixture.identifier} is mounted but should not, the template does not use keep_mounted policy"
             return fixture
 
-        tmp_prefix = '{0}_{1}'.format(fixture_template.identifier, '_')
-        fixture_directory = tempfile.mkdtemp(prefix=tmp_prefix)
-        os.rmdir(fixture_directory)
+        if fixture_template.mount_in_place:
+            fixture_directory = fixture_template.directory
+        else:
+            tmp_prefix = '{0}_{1}'.format(fixture_template.identifier, '_')
+            fixture_directory = tempfile.mkdtemp(prefix=tmp_prefix)
+            os.rmdir(fixture_directory)
 
         return Fixture.create_from_template(fixture_template, fixture_directory)
 
@@ -75,69 +82,128 @@ class FixtureEngine:
 
     def _teardown(self):
         for template, fixture in self.store.mounted_fixtures():
+            if fixture.state == State.Ready:
+                self.teardown_data(template, fixture, process_teardown=True)
+
             if fixture.state == State.Started:
-                self.stop(template, fixture, teardown=True)
-            self.unmount(template, fixture, teardown=True)
+                self.stop(template, fixture, process_teardown=True)
 
-    def start(self, template: FixtureTemplate, fixture: Fixture) -> None:
-        if self.store.is_running(template):
-            return
-
-        try:
-            self.plugin_engine.run(PluginEvent.starting, fixture)
-            self.hook_engine.run(HookEvent.starting, template)
-            self.store.fixture_started(fixture)
-        except PluginRuntimeError:
-            self.plugin_engine.release(PluginEvent.stopping, fixture)
-            self.plugin_engine.release(PluginEvent.unmounting, fixture)
-            if os.path.isdir(fixture.directory):
-                shutil.rmtree(fixture.directory, True)
-
-            raise
+            self.unmount(template, fixture, process_teardown=True)
 
     @contextlib.contextmanager
     def run(self, template: FixtureTemplate, fixture: Fixture):
         logger = get_logger()
         try:
             self.start(template, fixture)
+            self.setup_data(template, fixture)
             logger.debug(f'start fixture: {fixture.directory}')
             yield
 
         finally:
+            self.teardown_data(template, fixture)
+
             logger.debug(f'stop fixture : {fixture.directory}')
-            self.stop(template, fixture)
-
-    def stop(self, template: FixtureTemplate, fixture: Fixture, teardown: bool = False) -> None:
-        """
-
-        :param teardown: true when the fixture engine is tear downed at the end of the process usually
-        """
-        if template.keep_running is True and not teardown:
-            return
+            if fixture.state == State.Started:
+                self.stop(template, fixture)
 
+    def setup_data(self, template: FixtureTemplate, fixture: Fixture) -> None:
         try:
-            self.plugin_engine.run(PluginEvent.stopping, fixture)
-            self.hook_engine.run(HookEvent.stopping, template)
-            self.store.fixture_stopped(fixture)
-        except PluginRuntimeError:
+            self.plugin_engine.run(PluginEvent.setup_data, fixture)
+            self.hook_engine.run(HookEvent.setup_data, template)
+            fixture.setup()
+        except (PluginRuntimeError, HookRuntimeError):
+            self.plugin_engine.release(PluginEvent.teardown_data, fixture)
+            self.plugin_engine.release(PluginEvent.stopping, fixture)
             self.plugin_engine.release(PluginEvent.unmounting, fixture)
-            if os.path.isdir(fixture.directory):
+
+            # When a fixture is mount in place, Fixtup should preserve the original
+            # when error happens
+            if not template.mount_in_place and os.path.isdir(fixture.directory):
                 shutil.rmtree(fixture.directory, True)
 
             raise
 
-    def unmount(self, template: FixtureTemplate, fixture: Fixture, teardown: bool = False) -> None:
-        """
-
-        :param teardown: true when the fixture engine is tear downed at the end of the process usually
-        """
-        if (template.keep_mounted is True or template.keep_running is True) and not teardown:
+    def start(self, template: FixtureTemplate, fixture: Fixture) -> None:
+        if self.store.is_started(template):
             return
 
-        self.plugin_engine.run(PluginEvent.unmounting, fixture)
-        self.hook_engine.run(HookEvent.unmounting, template)
+        with with_cwd(fixture.directory):
+            try:
+                os.chdir(fixture.directory)
+                self.plugin_engine.run(PluginEvent.starting, fixture)
+                self.hook_engine.run(HookEvent.starting, template)
+                self.store.fixture_started(fixture)
+            except (PluginRuntimeError, HookRuntimeError):
+                self.plugin_engine.release(PluginEvent.stopping, fixture)
+                self.plugin_engine.release(PluginEvent.unmounting, fixture)
+
+                # When a fixture is mount in place, Fixtup should preserve the original
+                # when error happens
+                if not template.mount_in_place and os.path.isdir(fixture.directory):
+                    shutil.rmtree(fixture.directory, True)
+
+                raise
+
+    def stop(self, template: FixtureTemplate, fixture: Fixture, process_teardown: bool = False) -> None:
+        """
+
+        :param process_teardown: true when the fixture engine is tear downed at the end of the process usually
+        """
+        if template.keep_running is True and not process_teardown:
+            return
+
+        with with_cwd(fixture.directory):
+            try:
+                self.plugin_engine.run(PluginEvent.stopping, fixture)
+                self.hook_engine.run(HookEvent.stopping, template)
+                self.store.fixture_stopped(fixture)
+            except (PluginRuntimeError, HookRuntimeError):
+                self.plugin_engine.release(PluginEvent.unmounting, fixture)
+
+                # When a fixture is mount in place, Fixtup should preserve the original
+                # when error happens
+                if not template.mount_in_place and os.path.isdir(fixture.directory):
+                    shutil.rmtree(fixture.directory, True)
+
+                raise
+
+    def register_process_teardown(self):
+        """
+        register cleanup callbacks to unmount fixtures always mounted
+        at the end of the test runtime execution
+
+        Mounted fixtures are usually :
+
+        * fixture with shared policy
+        """
+        atexit.register(self.process_teardown_exit)
+        register_signal_handler(signal.SIGTERM, self.process_teardown_signal)
+        register_signal_handler(signal.SIGQUIT, self.process_teardown_signal)
+
+    def teardown_data(self, template, fixture, process_teardown: bool = False):
+        """
+
+        :param process_teardown: true when the fixture engine is tear downed at the end of the process usually
+        """
+        self.plugin_engine.run(PluginEvent.teardown_data, fixture)
+        self.hook_engine.run(HookEvent.teardown_data, fixture)
+        fixture.teardown()
+
+    def unmount(self, template: FixtureTemplate, fixture: Fixture, process_teardown: bool = False) -> None:
+        """
+
+        :param process_teardown: true when the fixture engine is tear downed at the end of the process usually
+        """
+        if (template.keep_mounted is True or template.keep_running is True) and not process_teardown:
+            return
+
+        with with_cwd(fixture.directory):
+            self.plugin_engine.run(PluginEvent.unmounting, fixture)
+            self.hook_engine.run(HookEvent.unmounting, template)
+
+        if not template.mount_in_place:
+            shutil.rmtree(fixture.directory, True)
 
-        shutil.rmtree(fixture.directory, True)
         self.store.fixture_unmounted(fixture)
 
     @contextlib.contextmanager
@@ -154,19 +220,6 @@ class FixtureEngine:
                 logger.debug(f'remove mounted fixture directory : {fixture.directory}')
                 self.unmount(template, fixture)
 
-    def register_process_teardown(self):
-        """
-        register cleanup callbacks to unmount fixtures always mounted
-        at the end of the test runtime execution
-
-        Mounted fixtures are usually :
-
-        * fixture with shared policy
-        """
-        atexit.register(self.process_teardown_exit)
-        register_signal_handler(signal.SIGTERM, self.process_teardown_signal)
-        register_signal_handler(signal.SIGQUIT, self.process_teardown_signal)
-
     def unregister_process_teardown(self):
         """
         unregister cleanup callbacks if cleanup was triggered
@@ -175,3 +228,5 @@ class FixtureEngine:
         atexit.unregister(self.process_teardown_exit)
         unregister_signal_handler(signal.SIGTERM, self.process_teardown_signal)
         unregister_signal_handler(signal.SIGQUIT, self.process_teardown_signal)
+
+
