@@ -5,6 +5,7 @@ import sys
 from typing import Optional
 
 from fixtup.entity.fixture_template import FixtureTemplate
+from fixtup.exceptions import HookRuntimeError
 from fixtup.hook.base import HookEngine, HookEvent
 from fixtup.lib.env import env_override
 from fixtup.logger import get_logger
@@ -39,14 +40,22 @@ class PythonHookEngine(HookEngine):
             If a developper set a breakpoint in its hook script, the python interpreter
             will stop on the breakpoint. This behavior makes it easier to debug a hook.
             """
-            subprocess.check_call([sys.executable, hook_script])
+            try:
+                subprocess.check_call([sys.executable, hook_script])
+            except subprocess.CalledProcessError as exception:
+                raise HookRuntimeError("Error on hook invocation",
+                                       template=template,
+                                       hook_event=event,
+                                       hook_script=hook_script)
 
 
 def script(event: 'HookEvent') -> Optional[str]:
     files = {
         HookEvent.mounting: os.path.join(".hooks", "hook_mounted.py"),
+        HookEvent.setup_data: os.path.join(".hooks", "hook_setup_data.py"),
         HookEvent.starting: os.path.join(".hooks", "hook_started.py"),
         HookEvent.stopping: os.path.join(".hooks", "hook_stopping.py"),
+        HookEvent.teardown_data: os.path.join(".hooks", "hook_teardown_data.py"),
         HookEvent.unmounting: os.path.join(".hooks", "hook_unmounting.py")
     }
 
