@@ -28,8 +28,10 @@ def event_to_function(event: PluginEvent) -> str:
         event.new_fixture: "on_new_fixture",
         event.mounting: "on_mounting",
         event.starting: "on_starting",
+        event.setup_data: "on_setup_data",
         event.stopping: "on_stopping",
         event.unmounting: "on_unmounting",
+        event.teardown_data: "on_teardown_data",
     }
 
     return _event_to_function[event]  # type: ignore
